
// Semantic //////////////////////////////////////////////
float4x4 matWorld 					: MATRIX_WORLD;
float4x4 matWorldViewProj 	: MATRIX_WORLDVIEWPROJ;


// Vars //////////////////////////////////////////////////
int OnOff 			= 1;
float Alpha 		= 1;
float Softness 	= .2;//.15;
float4 CamPos;
float3 PosCamera;


float3 lightSourceDir : LIGHT0_DIRECTION;
float4 ambientColor   : COLOR_AMBIENT;
float4 lightColor     : LIGHT0_COLOR;

float2 TexScale = float2(1.0f,1.0f); 	// Normals Tex Scale
const float2 OffsetF;									// Offset First
const float2 OffsetS;									// Offset Second
const float ViewSoft;									// View Soft
const half WaterInt;									// Water Intensity
const float FallOffSoft;							// FallOff Soft
const half FallOffInt;								// FallOff Intensity
const float3 WaterColor;							// Water Color
const float Refract;									// Refraction Intensity
const float Reflect;									// Reflection Intensity

static float3 cN;
static float4 cS;
static float cFV;
static float cFO;
static float2 ProjCoords;
static float3 nCamera;

// Depth Texture ///////////////////////////////////////////
texture DepthTexture;
sampler DEPTH 	= sampler_state {
		Texture   	= <DepthTexture>;
    AddressU 		= Clamp;
    AddressV 		= Clamp;
    MinFilter 	= Linear;
    MagFilter 	= Linear;
    MipFilter 	= Linear;
};

// Reflection & Reflaction /////////////////////////////////
const texture tReflection;
sampler TexReflection=sampler_state {
    Texture   = <tReflection>;
    ADDRESSU  = CLAMP;
    ADDRESSV  = CLAMP;
    ADDRESSW  = CLAMP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

const texture tScreen;
sampler TexScreen=sampler_state {
    Texture   = <tScreen>;
    ADDRESSU  = CLAMP;
    ADDRESSV  = CLAMP;
    ADDRESSW  = CLAMP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

// Normal Textures /////////////////////////////////////////
const texture tNormalW;
sampler TexNormalW=sampler_state {
    Texture   = <tNormalW>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};
const texture tNormalN;
sampler TexNormalN=sampler_state {
    Texture   = <tNormalN>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = LINEAR;
};


const texture FoamTexture;
sampler FoamT=sampler_state {
    Texture   = <FoamTexture>;
    ADDRESSU  = WRAP;
    ADDRESSV  = WRAP;
    ADDRESSW  = WRAP;
    MAGFILTER = LINEAR;
    MINFILTER = LINEAR;
    MIPFILTER = POINT;
};


struct VS_INPUT
{
	float4 Pos 				: POSITION;
	float2 Tex 				: TEXCOORD0;
	float3 Depth  		: TEXCOORD1;  
	float4 ParDepth		: TEXCOORD2;
	float4 P					: TEXCOORD3;
	float3 Normal 		: NORMAL;
	float3 Tangent 		: TANGENT;
	float3 Binormal 	: BINORMAL;

};

struct VS_OUTPUT
{
  float4 Pos 				: POSITION;
  float2 Tex 				: TEXCOORD0;
 	float3 Depth   		: TEXCOORD1;
 	float4 ParDepth		: TEXCOORD2;    
 	float4 P					: TEXCOORD3; 	
	float3 Normal 		: TEXCOORD4;
	float3 Tangent 		: TEXCOORD5;
	float3 Binormal 	: TEXCOORD6;
 		
};


float PhongSpecular(float3 normal, float3 viewDir, float specularDecay)
{
    float nDotL = dot(normal, lightSourceDir);
 
    float3 reflection = 2.0f * normal * nDotL + -lightSourceDir;
 
    reflection = normalize(reflection);
 
    float rdotV = saturate(dot(reflection, viewDir));
 
    return pow(rdotV, specularDecay);
}


static const int kernelSize = 13;
static const float downSize = 256.0f;


float2 pixelKernel[kernelSize] =
{
    { -6.0f / downSize, 0.0f },
    { -5.0f / downSize, 0.0f },
    { -4.0f / downSize, 0.0f },
    { -3.0f / downSize, 0.0f },
    { -2.0f / downSize, 0.0f },
    { -1.0f / downSize, 0.0f },
    {  0.0f,            0.0f },
    {  1.0f / downSize, 0.0f },
    {  2.0f / downSize, 0.0f },
    {  3.0f / downSize, 0.0f },
    {  4.0f / downSize, 0.0f },
    {  5.0f / downSize, 0.0f },
    {  6.0f / downSize, 0.0f },
};

float2 pixelKernelV[kernelSize] =
{
    { 0.0f, -6.0f / downSize },
    { 0.0f, -5.0f / downSize },
    { 0.0f, -4.0f / downSize },
    { 0.0f, -3.0f / downSize },
    { 0.0f, -2.0f / downSize },
    { 0.0f, -1.0f / downSize },
    { 0.0f,  0.0f            },
    { 0.0f,  1.0f / downSize },
    { 0.0f,  2.0f / downSize },
    { 0.0f,  3.0f / downSize },
    { 0.0f,  4.0f / downSize },
    { 0.0f,  5.0f / downSize },
    { 0.0f,  6.0f / downSize },
};


static const float blurWeights[kernelSize] = 
{
    0.002216,
    0.008764,
    0.026995,
    0.064759,
    0.120985,
    0.176033,
    0.199471,
    0.176033,
    0.120985,
    0.064759,
    0.026995,
    0.008764,
    0.002216,
};

// Wave functions //////////////////////////////////////////
float 		time				: CURRENT_TIME;
float WaveAmp = 0.04;
float WaveFreq = 15;

struct Wave {
  float freq;  // 2*PI / wavelength
  float amp;   // amplitude
  float phase; // speed * 2*PI / wavelength
  float2 dir;
};

#define NWAVES 2
Wave wave[NWAVES] = {
	{ 1.0, 1.0, 0.5, float2(-1, 0) },
	{ 2.0, 0.5, 1.3, float2(-0.7, 0.7) }	
};

float evaluateWave(Wave w, float2 pos, float t)
{
  return w.amp * sin( dot(w.dir, pos)*w.freq + t*w.phase);
}

float evaluateWaveDeriv(Wave w, float2 pos, float t)
{
  return w.freq*w.amp * cos( dot(w.dir, pos)*w.freq + t*w.phase);
}

void VS (	 in VS_INPUT IN, out VS_OUTPUT OUT )
{
	// Water anim	
  wave[0].freq = WaveFreq;
  wave[0].amp = WaveAmp;
  wave[1].freq = WaveFreq*2.0;
  wave[1].amp = WaveAmp*0.5;
  float4 Po = float4(IN.Pos.xyz,1.0);
  // sum waves	
  Po.y = 0.0;
  float ddy = 0.0;
  for(int i=0; i<NWAVES; i++) {
		Po.y += evaluateWave(wave[i], Po.xz, time);
		float deriv = evaluateWaveDeriv(wave[i], Po.xz, time);		
		ddy += deriv * wave[i].dir.y;		
	}
	IN.Pos.y = ddy; 	
     
	
	OUT.Pos 					= mul(IN.Pos, matWorldViewProj);
	OUT.Depth 				= mul(IN.Pos, matWorldViewProj).xyw;
	OUT.P				  		= mul(IN.Pos, matWorld);
	OUT.Tex 					= IN.Tex;		
	OUT.ParDepth  		= OUT.Pos;
	OUT.Normal				= normalize(mul(IN.Normal,matWorld));
	OUT.Tangent				= normalize(mul(IN.Tangent,matWorld));
	OUT.Binormal			= normalize(mul(IN.Binormal,matWorld));

};

float4 PS (	in VS_OUTPUT IN ) : COLOR 
{
	// Water FX //////////////////////////////////////////////////////////////////
	cN						= tex2D( TexNormalW, IN.Tex / TexScale + OffsetF ) * 0.5f;
	cN			 	 	 += tex2D( TexNormalN, IN.Tex / TexScale + OffsetS ) * 0.5f;
	cN						= normalize( cN * 2.0f - 1.0f );

	ProjCoords		= ( 0.5 * IN.Depth.xy / IN.Depth.z + 0.5 );	

	// Reflection Blur
	//float4 Color 	= tex2D( TexReflection, ProjCoords + cN.xy * Reflect );	
  float4 Color = 0;
  for(int i = 0; i < kernelSize; i++) {   
		Color += tex2D(TexReflection, (ProjCoords + cN.xy * Reflect) + (pixelKernel[i].xy)) * blurWeights[i];	
		Color += tex2D(TexReflection, (ProjCoords + cN.xy * Reflect) + (pixelKernelV[i].xy)) * blurWeights[i];
	}
  Color *= 0.5;
    	
	// Refraction
	//cS						= tex2D( TexScreen, ProjCoords + cN.xy * Refract );
	
	PosCamera			= normalize(CamPos-IN.P);
	nCamera				= normalize(float3(dot(PosCamera,IN.Tangent), dot(PosCamera,IN.Binormal),dot(PosCamera,IN.Normal)));
	
	cFV						= pow(max(dot(cN,nCamera),0.0f),ViewSoft)*WaterInt;
	cFO						= pow(max(dot(cN,nCamera),0.0f),FallOffSoft)*FallOffInt;

	Color				 *= (1.0f-cFV)+cS*cFV; 
	Color				 += Color * (1.0f-cFO) * FallOffInt * float4(WaterColor,1);
	
	float3 normalWorld = normalize((float3(0, 1, 0) * cN.z) + (cN.x * float3(0, 0, 1) + cN.y * float3(-1, 0, 0)));
	float specular = PhongSpecular(normalWorld, -PosCamera, 512);
	Color.rgb += specular;

	// Soft edges ////////////////////////////////////////////////////////////////
	IN.Depth.xz 			= -IN.Depth.xz;	
	float SceneDepth	= tex2D(DEPTH, 0.5 * IN.Depth.xy / IN.Depth.z + 0.5);

	float Dist 				= distance(IN.P,CamPos);				
	float CurDepth		= IN.ParDepth.z / IN.ParDepth.w;	
	float DepthDiff		= ( SceneDepth - CurDepth ) * Dist * ( Softness * Dist );
	DepthDiff					= saturate( DepthDiff );

	Color.a = ( DepthDiff * Alpha );

	float4 Foam = tex2D( FoamT, (IN.Tex * 70) + ( OffsetF / 10 ) + ( cN.xy / 100 ) ) - DepthDiff; // * 1.5	
	Foam = ambientColor * Foam + (Foam * saturate(dot(IN.Normal, -lightSourceDir)) * lightColor);	
	Color.rgb += saturate(Foam);

	return Color;
};

technique Soft { 
	pass p {
		VertexShader 	= compile vs_2_0 VS();
		PixelShader 	= compile ps_3_0 PS();
	}
}

